// import { AuthMail, DataMail, EsquemaTrabajo } from "./types";
import { DEFAULT_TURN, DEFAULT_UTC } from "./values";
import { networkInterfaces, totalmem,cpus }  from 'os';
import getMAC from 'getmac'
import { createLogger, format, transports } from "winston";
import moment from "moment-timezone";
import nodemailer from "nodemailer";

export namespace Utils {

    /**
     * @param dni 
     * Dado un RUN sin el DV, obtiene el dígito verificador
     */
    export const getDV = (dni: number) => {
        const adni = dni;
        let M=0,S=1;
        for(;dni;dni=Math.floor(dni/10)) {
          S = (S+dni%10*(9-M++%6))%11;
        }
        const dv = S ? `${S-1}` : 'K';
        return `${adni.toString().trim()}-${dv.toString().trim()}`
    }


    export const logger = createLogger({
        level: 'info',
        format: format.combine(
            format(info => {
                info.level = info.level.toUpperCase();
                return info;
            })(),
            format.colorize(),
            format.timestamp({
              format: 'YYYY-MM-DD HH:mm:ss'
            }),
            format.printf(info => {
              return `${info.timestamp} - ${info.level}: ${info.message}`;
            })
          ),
        transports: [new transports.Console()]
    });

    export const isJSONString = (cadena: string) => {

        cadena = typeof cadena !== "string" ? JSON.stringify(cadena) : cadena;
        try {
            cadena = JSON.parse(cadena);
        } catch (e) {
            return false;
        }
        if (typeof cadena === "object" && cadena !== null) {
            return true;
        }
        return false;
    
    }

    export const stringToAccesgroup = (str: string) => {
    
        if (!str) return undefined;

        return  str.toUpperCase()
                    .replace(/[.,\/#!$%\^&\*;:{}=\-_`~() ]/g, '')
                    .normalize("NFD")
                    .replace(/[\u0300-\u036f]/g, "");
    
    }
    
    export const momento = () => {
        return moment().tz("America/Santiago")
    }
 

    export const sendMailTo = async (data: Types.DataMail, auth: Types.AuthMail) => {

        try {
            const host = `${auth.host}`;
            const email = `${auth.user}`;
            const pass = `${auth.passwd}`;
            const port = Number(auth.port);

            const mailTransport = nodemailer.createTransport({
                host,
                port,
                auth: { user: email, pass },
                tls: { rejectUnauthorized: false }
            });
            
            await mailTransport.sendMail({
                from: email,
                to: data.mail,
                replyTo: email,
                subject: `${data.subject}`,
                html: data.body
            });
        } catch (error) {
            logger.error(JSON.stringify(error));
        }
      }

      
}



export namespace Turnos {

    const normalizeHourMin = (h: number) => (Number(h)<10)?`0${h}`:h;

    const normalizeViewTime = (jornada: string) => {

        const iniFin = jornada.split('-');
        const ini = iniFin[0].split('.');
        const fin = iniFin[1].split('.');

        const hourIni = normalizeHourMin(Number(ini[0]));
        const minIni = ini.length > 1 ? normalizeHourMin(Number(ini[1])) : '00';

        const hourFin = normalizeHourMin(Number(fin[0]));
        const minFin = fin.length > 1 ? normalizeHourMin(Number(fin[1])) : '00';

        return {
            inicio: {
                timeView: `${hourIni}:${minIni}`,
                hour: Number(hourIni),
                min: Number(minIni),
            },
            fin: {
                timeView: `${hourFin}:${minFin}`,
                hour: Number(hourFin),
                min: Number(minFin)
            }
        }
    }

    /**
     * @param esquemas 
     * @param utc 
     * @param daySelected 
     * Determina el inicio y fin de turno en horario UTC 0 y tz nacional basado en un esquema de trabajo.
     */
    export const timesTurno = (esquemas: Types.EsquemaTrabajo[], utc: number, daySelected?: Date) => {

        const auxDate = new Date();
        const difHoraria = utc || DEFAULT_UTC;
        let fecha = daySelected || new Date(auxDate.setHours(auxDate.getHours() - difHoraria));

        const numberOfDay = fecha.getDay()
        let turno = DEFAULT_TURN;
        for(const eachTurno of esquemas) {
            if (eachTurno.dias.map(it => it === 7 ? 0 : it ).findIndex(it => it === numberOfDay) >= 0) {
                turno = `${eachTurno.horaInicio}-${eachTurno.horaFin}`;
                break;
            }
        }

        const { inicio, fin } = normalizeViewTime(turno);
        const horaInicio = Number(inicio.hour);
        const horaFin = Number(fin.hour);
        const horaActual = new Date(new Date().getTime() - difHoraria * 60 * 60 * 1000).getHours();

        if ( horaInicio > horaFin && horaActual < horaFin ) {
            fecha = new Date(fecha.setDate(fecha.getDate() - 1));
        }

        const miliSecondUTCIni = new Date(fecha).setHours(horaInicio + difHoraria, inicio.min, 0, 0);
        const miliSecondTZIni = new Date(fecha).setHours(horaInicio, inicio.min, 0, 0);

        if (horaInicio > horaFin)  {
            // Si ha cambiado de día, fin del turno siempre será el dia siguiente
            fecha = new Date(fecha.setDate(fecha.getDate() + 1));
        }

        const miliSecondUTCFin = new Date(fecha).setHours(horaFin + difHoraria, fin.min, 0, 0);
        const miliSecondTZFin = new Date(fecha).setHours(horaFin, fin.min, 0, 0);

        const pt1 = `${inicio.timeView} (${normalizeHourMin(new Date(miliSecondTZIni).getDate())}/${normalizeHourMin(new Date(miliSecondTZIni).getMonth() + 1) })`;
        const pt2 = `${fin.timeView} (${normalizeHourMin(new Date(miliSecondTZFin).getDate())}/${normalizeHourMin(new Date(miliSecondTZFin).getMonth() + 1) })`

        return {
            turnoIni: {
                UTCTurno: new Date(miliSecondUTCIni).toISOString(),
                TZTurno: new Date(miliSecondTZIni).toISOString(),
                timeView: inicio.timeView
            },
            turnoFin: {
                UTCTurno: new Date(miliSecondUTCFin).toISOString(),
                TZTurno: new Date(miliSecondTZFin).toISOString(),
                timeView: fin.timeView
            },
            turnoString:  `${pt1} - ${pt2}`
        }

    }

    /**
     * 
     * @param begFin
     * @returns 
     * Determina una fecha según un numeral. Por ej, el primer sabado de abril: begFin: 1-6-4
     */
    export const getNumeralDayOfMonth = (begFin: { numeral: number, day: number, month: number, year: number}) => {
    
        const { month, year, day, numeral } = begFin;
        
        if ( month > 12 || month < 1 ) return 'input error';
        if ( day > 7 || day < 1 ) return 'input error';
        if ( numeral > 5 || numeral < 1) return 'input error';
    
        let dateFounded = [];
        let dateInit = new Date(`${year}-${month}-01`)
        while (month === dateInit.getMonth()+1) {
            if (dateInit.getDay() === day) {
                dateFounded.push(dateInit);
            }
            dateInit = new Date(new Date(dateInit).setDate(dateInit.getDate() + 1));
        }
        return dateFounded[numeral-1];
    }

    export const getMonthsDST = (timezone: string) => {
        switch (timezone) {
            case 'America/Santiago':
                return {ini: 9, fin: 4};
            default:
                return {ini: 0, fin: 0};
        }
    }
}


export namespace Machine {

    export const totalRam = () => { 
        return (Math.round(totalmem()/1000000)).toString()+'MB';          
      }
      
      export const getCpu = () => {
        return cpus()[0].model;
      }
      
      export const getCpuSpeed = () => {
        return cpus()[0].speed / 1000 + 'GHZ';
      }
      
      export const getMacAddress = () => {
        return getMAC();
      }

      export const getIp = () => {
        const interfaces = networkInterfaces();
        const addresses = [];
    
        Object.keys(interfaces).forEach((netInterface) => {
            interfaces[netInterface].forEach((interfaceObject) => {
                if (interfaceObject.family === 'IPv4' && !interfaceObject.internal) {
                    addresses.push(interfaceObject.address);
                }
            });
        });
        return (addresses[0]);
        
    }

}


export namespace Types {

    export interface DataMail { 
        mail: string; 
        subject: string; 
        body: string 
    }
    
    export interface AuthMail { 
        user: string; 
        passwd: string; 
        host: string;
        port: string;
    }
    
    export interface EsquemaTrabajo {
        horaInicio: string;
        horaFin: string;
        dias: number[];
    }

}